@@ -73,10 +73,9 @@ sample.Spatial = function(x, n, type, bb = bbox(x), offset = runif(nrow(bb)),
 }
 setMethod("spsample", signature(x = "Spatial"), sample.Spatial)
 
-sample.Line = function(x, n, type, offset = runif(1),
-		proj4string=CRS(as.character(NA)), ...) {
+sample.Line = function(x, n, type, offset = runif(1), proj4string = CRS(as.character(NA)), ...) {
+	offset = offset[1]
 	if (missing(n)) n <- as.integer(NA)
-#cat("n in sample.Line", n, "\n")
 	cc = coordinates(x)
 	dxy = apply(cc, 2, diff)
 	if (inherits(dxy, "matrix"))
@@ -97,14 +96,45 @@ sample.Line = function(x, n, type, offset = runif(1),
 	int = findInterval(pts, csl, all.inside = TRUE)
 	where = (pts - csl[int])/diff(csl)[int]
 	xy = cc[int,] + where * (cc[int+1,] - cc[int,])
-#	SpatialPoints(xy, CRS(proj4string(x)))
-	SpatialPoints(xy, proj4string=proj4string)
+	SpatialPoints(xy, proj4string)
 }
 setMethod("spsample", signature(x = "Line"), sample.Line)
 
+sample.Lines = function(x, n, type, offset = runif(1), ...) {
+	L = x@Lines
+	lengths = sapply(L, function(x) LineLength(x@coords))
+	nrs = round(lengths / sum(lengths) * n)
+	ret = vector("list", sum(nrs > 0))
+	j = 1
+	for (i in 1:length(L)) {
+		if (nrs[i] > 0) {
+			ret[[j]] = spsample(L[[i]], nrs[i], type = type, offset = offset, ...)
+			j = j+1
+		}
+	}
+	do.call("rbind", ret)
+}
+setMethod("spsample", signature(x = "Lines"), sample.Lines)
+
+sample.SpatialLines = function(x, n, type, offset = runif(1), ...) {
+	lengths = SpatialLinesLengths(x)
+	nrs = round(lengths / sum(lengths) * n)
+	ret = vector("list", sum(nrs > 0))
+	j = 1
+	for (i in 1:length(lengths)) {
+		if (nrs[i] > 0) {
+			ret[[j]] = spsample(x@lines[[i]], nrs[i], type = type, offset = offset, ...)
+			j = j+1
+		}
+	}
+	ret = do.call("rbind", ret)
+	proj4string(ret) = CRS(proj4string(x))
+	ret
+}
+setMethod("spsample", signature(x = "SpatialLines"), sample.SpatialLines)
+
 sample.Polygon = function(x, n, type = "random", bb = bbox(x),
 	offset = runif(2), proj4string=CRS(as.character(NA)), iter=4, ...) {
-#...) {
 	if (missing(n)) n <- as.integer(NA)
 #cat("n in sample.Polygon", n, "\n")
 	area = getPolygonAreaSlot(x)
@@ -146,12 +176,8 @@ sample.Polygon = function(x, n, type = "random", bb = bbox(x),
 setMethod("spsample", signature(x = "Polygon"), sample.Polygon)
 
 sample.Polygons = function(x, n, type = "random", bb = bbox(x),
-		offset = runif(2), 
-#...) {
-proj4string=CRS(as.character(NA)), iter=4, ...) {
-	#stop("not functioning yet...")
+		offset = runif(2), proj4string=CRS(as.character(NA)), iter=4, ...) {
 	if (missing(n)) n <- as.integer(NA)
-#cat("n in sample.Polygons", n, "\n")
 	area = sapply(getPolygonsPolygonsSlot(x), getPolygonAreaSlot) # also available for Polygons!
 	if (sum(area) == 0.0)
 		# distribute n over the lines, according to their length?
@@ -175,8 +201,7 @@ proj4string=CRS(as.character(NA)), iter=4, ...) {
 	    for (i in seq(along=ptsres)) {
 		if (smple[i]) ptsres[[i]] <- sample.Polygon(
 		    x=pls[[i]], n=round(n*(area[i]/sum_area)), 
-		    type = type, offset = offset, proj4string=proj4string, 
-		    iter=iter)
+		    type = type, offset = offset, iter=iter)
 	    }
 	    crds <- do.call("rbind", lapply(ptsres, function(x) 
 	        if (!is.null(x)) coordinates(x)))
@@ -184,7 +209,7 @@ proj4string=CRS(as.character(NA)), iter=4, ...) {
 	    else {
 	        pts <- SpatialPoints(crds, proj4string=proj4string)
 	        id = overlay(pts, SpatialPolygons(list(x), 
-		    proj4string=proj4string))
+				proj4string=proj4string))
 	        Not_NAs <- !is.na(id)
 	        if (!any(Not_NAs)) res <- NULL
 	        else res <- pts[which(Not_NAs)]
@@ -224,6 +249,7 @@ sample.Spatial(as(x, "Spatial"), round(n * bb.area/area), type=type, offset = of
 	if (type == "random")
 	    if (!is.null(res) && n < nrow(res@coords)) 
 		res <- res[sample(nrow(res@coords), n)]
+	proj4string(res) = CRS(proj4string(x))
 	res
 }
 setMethod("spsample", signature(x = "SpatialPolygons"), sample.SpatialPolygons)
@@ -290,12 +316,13 @@ genHexGrid <- function(dx, ll = c(0, 0), ur = c(1, 1)) {
         data.frame(x = x, y = y)
 }
 
-genPolyList <- function(hexGrid) {
+genPolyList <- function(hexGrid, dx) {
 	# EJP; changed:
 	# how to figure out dx from a grid? THK suggested:
         #dx <- hexGrid$x[2] - hexGrid$x[1]
 	# and the following will also not allways work:
-	dx = 2 * min(diff(sort(unique(hexGrid$x))))
+	if (missing(dx))
+		dx = 2 * min(diff(sort(unique(hexGrid$x))))
 	dy <- dx / sqrt(3)
 
 	x.offset <- c(-dx / 2, 0, dx / 2, dx / 2, 0, -dx / 2, -dx / 2)
@@ -307,17 +334,15 @@ genPolyList <- function(hexGrid) {
 	ret = lapply(1:length(hexGrid$x), f)
 }
 
-as.SpatialPolygons.HexGrid = function(hex) {
-	ret = genPolyList(data.frame(coordinates(hex)))
+# EJP, added:
+HexPoints2SpatialPolygons = function(hex, dx) {
+	ret = genPolyList(data.frame(coordinates(hex)), dx = dx)
 	npoly = length(ret)
 	Srl <- vector(mode="list", length=npoly)
 	IDS = paste("ID", 1:npoly, sep="")
 	for (i in 1:npoly)
 		Srl[[i]] = Polygons(list(Polygon(ret[[i]])), IDS[i])
-	res <- as.SpatialPolygons.PolygonsList(Srl, proj4string=CRS(proj4string(hex)))
+	res <- as.SpatialPolygons.PolygonsList(Srl, 
+		proj4string=CRS(proj4string(hex)))
 	res
 }
-
-# xy <- genHexGrid(0.05)
-# plot(xy, asp = 1, axes = F, pch = 19, xlab = NA, ylab = NA, cex = 0.5)
-# lapply(genPolyList(xy), polygon, xpd = NA, ypd = NA)
